@@ -27,6 +27,9 @@
 // MySQL variables funcs
 
 #include "sql_priv.h"
+#include <sql_plugin.h>
+#include <mysql/plugin.h>
+
 class sys_var;
 class set_var;
 class THD;
@@ -38,9 +41,7 @@ int wsrep_init_vars();
 #define DEFAULT_ARGS (THD* thd, enum_var_type var_type)
 #define INIT_ARGS    (const char* opt)
 
-struct system_variables;
-bool wsrep_causal_reads_update(struct system_variables *sv);
-
+extern bool wsrep_causal_reads_update         UPDATE_ARGS;
 extern bool wsrep_on_update                  UPDATE_ARGS;
 extern bool wsrep_sync_wait_update           UPDATE_ARGS;
 extern bool wsrep_start_position_check       CHECK_ARGS;
@@ -89,6 +90,9 @@ extern bool wsrep_slave_threads_update       UPDATE_ARGS;
 extern bool wsrep_desync_check               CHECK_ARGS;
 extern bool wsrep_desync_update              UPDATE_ARGS;
 
+extern bool wsrep_max_ws_size_check          CHECK_ARGS;
+extern bool wsrep_max_ws_size_update         UPDATE_ARGS;
+
 #else  /* WITH_WSREP */
 
 #define WSREP_NONE
