@@ -25,7 +25,6 @@
 #define nullptr 0
 
 bool IsNum(PSZ s);
-bool MakeSelector(PGLOBAL g, PFIL fp, PSTRG s);
 
 /* --------------------------- Class JNCOL --------------------------- */
 
@@ -124,13 +123,12 @@ void JMgoConn::AddJars(PSTRG jpop, char sep)
 #if defined(DEVELOPMENT)
 	if (m_Version == 2) {
 		jpop->Append(sep);
-//	jpop->Append("C:/Eclipse/workspace/MongoWrap2/bin");
+		jpop->Append("C:/Eclipse/workspace/MongoWrap2/bin");
 		jpop->Append(sep);
 		jpop->Append("C:/mongo-java-driver/mongo-java-driver-2.13.3.jar");
 	} else {
 		jpop->Append(sep);
-//	jpop->Append("C:/Eclipse/workspace/MongoWrap3/bin");
-//	jpop->Append("C:/Program Files/MariaDB 10.1/lib/plugin/JavaWrappers.jar");
+		jpop->Append("C:/Eclipse/workspace/MongoWrap3/bin");
 		jpop->Append(sep);
 		jpop->Append("C:/mongo-java-driver/mongo-java-driver-3.4.2.jar");
 	} // endif m_Version
@@ -240,7 +238,6 @@ bool JMgoConn::MakeCursor(PGLOBAL g, PTDB tdbp, PCSZ options,
 	PSZ   jp;
 	PCSZ  op = NULL, sf = NULL, Options = options;
 	PSTRG s = NULL;
-	PFIL  filp = tdbp->GetFilter();
 
 	if (Options && !stricmp(Options, "all")) {
 		Options = NULL;
@@ -254,7 +251,7 @@ bool JMgoConn::MakeCursor(PGLOBAL g, PTDB tdbp, PCSZ options,
 			all = true;
 
 	if (pipe && Options) {
-		if (trace(1))
+		if (trace)
 			htrc("Pipeline: %s\n", Options);
 
 		p = strrchr(Options, ']');
@@ -267,10 +264,10 @@ bool JMgoConn::MakeCursor(PGLOBAL g, PTDB tdbp, PCSZ options,
 
 		s = new(g) STRING(g, 1023, (PSZ)Options);
 
-		if (filp) {
+		if (tdbp->GetFilter()) {
 			s->Append(",{\"$match\":");
 
-			if (MakeSelector(g, filp, s)) {
+			if (tdbp->GetFilter()->MakeSelector(g, s)) {
 				strcpy(g->Message, "Failed making selector");
 				return NULL;
 			} else
@@ -312,20 +309,20 @@ bool JMgoConn::MakeCursor(PGLOBAL g, PTDB tdbp, PCSZ options,
 		*(char*)p = ']';		 // Restore Colist for discovery
 		p = s->GetStr();
 
-		if (trace(33))
+		if (trace)
 			htrc("New Pipeline: %s\n", p);
 
 		return AggregateCollection(p);
 	} else {
-		if (filter || filp) {
-			if (trace(1)) {
+		if (filter || tdbp->GetFilter()) {
+			if (trace) {
 				if (filter)
 					htrc("Filter: %s\n", filter);
 
-				if (filp) {
+				if (tdbp->GetFilter()) {
 					char buf[512];
 
-					filp->Prints(g, buf, 511);
+					tdbp->GetFilter()->Prints(g, buf, 511);
 					htrc("To_Filter: %s\n", buf);
 				} // endif To_Filter
 
@@ -334,11 +331,11 @@ bool JMgoConn::MakeCursor(PGLOBAL g, PTDB tdbp, PCSZ options,
 			s = new(g) STRING(g, 1023, (PSZ)filter);
 			len = s->GetLength();
 
-			if (filp) {
+			if (tdbp->GetFilter()) {
 				if (filter)
 					s->Append(',');
 
-				if (MakeSelector(g, filp, s)) {
+				if (tdbp->GetFilter()->MakeSelector(g, s)) {
 					strcpy(g->Message, "Failed making selector");
 					return NULL;
 				}	// endif Selector
@@ -346,7 +343,7 @@ bool JMgoConn::MakeCursor(PGLOBAL g, PTDB tdbp, PCSZ options,
 				tdbp->SetFilter(NULL);     // Not needed anymore
 			} // endif To_Filter
 
-			if (trace(33))
+			if (trace)
 				htrc("selector: %s\n", s->GetStr());
 
 			s->Resize(s->GetLength() + 1);
@@ -355,7 +352,7 @@ bool JMgoConn::MakeCursor(PGLOBAL g, PTDB tdbp, PCSZ options,
 
 		if (!all) {
 			if (Options && *Options) {
-				if (trace(1))
+				if (trace)
 					htrc("options=%s\n", Options);
 
 				op = Options;
@@ -522,7 +519,7 @@ PSZ JMgoConn::GetDocument(void)
 		jdc = (jstring)env->CallObjectMethod(job, getdocid);
 
 		if (jdc)
-			doc = (PSZ)GetUTFString(jdc);
+			doc = (PSZ)env->GetStringUTFChars(jdc, (jboolean)false);
 
 	} // endif getdocid
 
@@ -690,7 +687,7 @@ jobject JMgoConn::MakeDoc(PGLOBAL g, PJNCOL jcp)
 
 		} // endif Jncolp
 
-	return parent;
+		return parent;
 } // end of MakeDoc
 
 /***********************************************************************/
@@ -751,7 +748,7 @@ int JMgoConn::DocUpdate(PGLOBAL g, PTDB tdbp)
 
 	jlong ar = env->CallLongMethod(job, updateid, upd);
 
-	if (trace(1))
+	if (trace)
 		htrc("DocUpdate: ar = %ld\n", ar);
 
 	if (Check((int)ar)) {
@@ -770,7 +767,7 @@ int JMgoConn::DocDelete(PGLOBAL g, bool all)
 	int   rc = RC_OK;
 	jlong ar = env->CallLongMethod(job, deleteid, all);
 
-	if (trace(1))
+	if (trace)
 		htrc("DocDelete: ar = %ld\n", ar);
 
 	if (Check((int)ar)) {
@@ -807,7 +804,7 @@ PSZ JMgoConn::GetColumnValue(PSZ path)
 		fn = (jstring)env->CallObjectMethod(job, objfldid, jn);
 
 		if (fn)
-			fld = (PSZ)GetUTFString(fn);
+			fld = (PSZ)env->GetStringUTFChars(fn, (jboolean)false);
 
 	}	// endif objfldid
 
