@@ -1,7 +1,7 @@
 /************* Value C++ Functions Source Code File (.CPP) *************/
-/*  Name: VALUE.CPP  Version 2.5                                       */
+/*  Name: VALUE.CPP  Version 2.8                                       */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          2001-2015    */
+/*  (C) Copyright to the author Olivier BERTRAND          2001-2017    */
 /*                                                                     */
 /*  This file contains the VALUE and derived classes family functions. */
 /*  These classes contain values of different types. They are used so  */
@@ -60,7 +60,7 @@
 #define CheckType(V)    if (Type != V->GetType()) { \
     PGLOBAL& g = Global; \
     strcpy(g->Message, MSG(VALTYPE_NOMATCH)); \
-    longjmp(g->jumper[g->jump_level], Type); }
+    throw Type;
 #else
 #define CheckType(V)
 #endif
@@ -94,12 +94,12 @@ PSZ strlwr(PSZ s);
 /*  OUT minus: Set to true if the number is negative                   */
 /*  Returned val: The resulting number                                 */
 /***********************************************************************/
-ulonglong CharToNumber(char *p, int n, ulonglong maxval,
+ulonglong CharToNumber(const char *p, int n, ulonglong maxval,
                        bool un, bool *minus, bool *rc)
 {
-  char     *p2;
-  uchar     c;
-  ulonglong val;
+  const char *p2;
+  uchar       c;
+  ulonglong   val;
 
   if (minus) *minus = false;
   if (rc) *rc = false;
@@ -119,6 +119,7 @@ ulonglong CharToNumber(char *p, int n, ulonglong maxval,
         if (minus) *minus = true;
       } // endif Unsigned
 
+			// Fall through
     case '+':
       p++;
       break;
@@ -138,9 +139,9 @@ ulonglong CharToNumber(char *p, int n, ulonglong maxval,
 /***********************************************************************/
 /*  GetTypeName: returns the PlugDB internal type name.                */
 /***********************************************************************/
-PSZ GetTypeName(int type)
+PCSZ GetTypeName(int type)
   {
-  PSZ name;
+  PCSZ name;
 
   switch (type) {
     case TYPE_STRING: name = "CHAR";     break;
@@ -175,7 +176,7 @@ int GetTypeSize(int type, int len)
     case TYPE_DOUBLE: len = sizeof(double);     break;
     case TYPE_TINY:   len = sizeof(char);       break;
     case TYPE_PCHAR:  len = sizeof(char*);      break;
-    default:          len = 0;
+    default:          len = -1;
     } // endswitch type
 
   return len;
@@ -184,9 +185,9 @@ int GetTypeSize(int type, int len)
 /***********************************************************************/
 /*  GetFormatType: returns the FORMAT character(s) according to type.  */
 /***********************************************************************/
-char *GetFormatType(int type)
+const char *GetFormatType(int type)
   {
-  char *c = "X";
+  const char *c = "X";
 
   switch (type) {
     case TYPE_STRING: c = "C"; break;
@@ -235,6 +236,7 @@ bool IsTypeChar(int type)
   switch (type) {
     case TYPE_STRING:
     case TYPE_DECIM:
+		case TYPE_BIN:
       return true;
     } // endswitch type
 
@@ -370,7 +372,7 @@ PVAL AllocateValue(PGLOBAL g, void *value, short type, short prec)
 /*  Allocate a variable Value according to type, length and precision. */
 /***********************************************************************/
 PVAL AllocateValue(PGLOBAL g, int type, int len, int prec,
-                   bool uns, PSZ fmt)
+                   bool uns, PCSZ fmt)
   {
   PVAL valp;
 
@@ -558,6 +560,38 @@ bool VALUE::Compute(PGLOBAL g, PVAL *, int, OPVAL)
   return true;
   } // end of Compute
 
+/***********************************************************************/
+/*  Make file output of an object value.                               */
+/***********************************************************************/
+void VALUE::Printf(PGLOBAL g, FILE *f, uint n)
+{
+	char m[64], buf[64];
+
+	memset(m, ' ', n);                             /* Make margin string */
+	m[n] = '\0';
+
+	if (Null)
+		fprintf(f, "%s<null>\n", m);
+	else
+		fprintf(f, "%s%s\n", m, GetCharString(buf));
+
+} /* end of Printf */
+
+/***********************************************************************/
+/*  Make string output of an object value.                             */
+/***********************************************************************/
+void VALUE::Prints(PGLOBAL g, char *ps, uint z)
+{
+	char *p, buf[64];
+
+	if (Null)
+		p = strcpy(buf, "<null>");
+	else
+		p = GetCharString(buf);
+
+  strncpy(ps, p, z);
+} // end of Prints
+
 /* -------------------------- Class TYPVAL ---------------------------- */
 
 /***********************************************************************/
@@ -632,7 +666,7 @@ bool TYPVAL<TYPE>::SetValue_pval(PVAL valp, bool chktype)
     if (chktype && Type != valp->GetType())
       return true;
 
-    if (!(Null = valp->IsNull() && Nullable))
+    if (!(Null = (valp->IsNull() && Nullable)))
       Tval = GetTypedValue(valp);
     else
       Reset();
@@ -682,7 +716,7 @@ uchar TYPVAL<uchar>::GetTypedValue(PVAL valp)
 /*  TYPVAL SetValue: convert chars extracted from a line to TYPE value.*/
 /***********************************************************************/
 template <class TYPE>
-bool TYPVAL<TYPE>::SetValue_char(char *p, int n)
+bool TYPVAL<TYPE>::SetValue_char(const char *p, int n)
   {
   bool      rc, minus;
   ulonglong maxval = MaxVal();
@@ -704,7 +738,7 @@ bool TYPVAL<TYPE>::SetValue_char(char *p, int n)
   } // end of SetValue
 
 template <>
-bool TYPVAL<double>::SetValue_char(char *p, int n)
+bool TYPVAL<double>::SetValue_char(const char *p, int n)
   {
   if (p && n > 0) {
     char buf[64];
@@ -732,7 +766,7 @@ bool TYPVAL<double>::SetValue_char(char *p, int n)
 /*  TYPVAL SetValue: fill a typed value from a string.                 */
 /***********************************************************************/
 template <class TYPE>
-void TYPVAL<TYPE>::SetValue_psz(PSZ s)
+void TYPVAL<TYPE>::SetValue_psz(PCSZ s)
   {
   if (s) {
     SetValue_char(s, (int)strlen(s));
@@ -792,19 +826,29 @@ uchar TYPVAL<uchar>::GetTypedValue(PVBLK blk, int n)
 
 /***********************************************************************/
 /*  TYPVAL SetBinValue: with bytes extracted from a line.              */
+/*  Currently only used reading column of binary files.                */
 /***********************************************************************/
 template <class TYPE>
 void TYPVAL<TYPE>::SetBinValue(void *p)
-  {
-  Tval = *(TYPE *)p;
-  Null = false;
-  } // end of SetBinValue
+{
+#if defined(UNALIGNED_OK)
+	// x86 can cast non-aligned memory directly
+	Tval = *(TYPE *)p;
+#else
+	// Prevent unaligned memory access on MIPS and ArmHF platforms.
+	// Make use of memcpy instead of straight pointer dereferencing.
+	// Currently only used by WriteColumn of binary files.
+	// From original author: Vicentiu Ciorbaru <vicentiu@mariadb.org>
+	memcpy(&Tval, p, sizeof(TYPE));
+#endif
+	Null = false;
+} // end of SetBinValue
 
 /***********************************************************************/
 /*  GetBinValue: fill a buffer with the internal binary value.         */
 /*  This function checks whether the buffer length is enough and       */
 /*  returns true if not. Actual filling occurs only if go is true.     */
-/*  Currently used by WriteColumn of binary files.                     */
+/*  Currently only used writing column of binary files.                */
 /***********************************************************************/
 template <class TYPE>
 bool TYPVAL<TYPE>::GetBinValue(void *buf, int buflen, bool go)
@@ -819,7 +863,16 @@ bool TYPVAL<TYPE>::GetBinValue(void *buf, int buflen, bool go)
 //#endif
 
   if (go)
-    *(TYPE *)buf = Tval;
+#if defined(UNALIGNED_OK)
+		// x86 can cast non-aligned memory directly
+		*(TYPE *)buf = Tval;
+#else
+		// Prevent unaligned memory access on MIPS and ArmHF platforms.
+		// Make use of memcpy instead of straight pointer dereferencing.
+		// Currently only used by WriteColumn of binary files.
+		// From original author: Vicentiu Ciorbaru <vicentiu@mariadb.org>
+		memcpy(buf, &Tval, sizeof(TYPE));
+#endif
 
   Null = false;
   return false;
@@ -989,7 +1042,7 @@ uchar TYPVAL<uchar>::MinMaxVal(bool b)
   {return (b) ? UINT_MAX8 : 0;}
 
 /***********************************************************************/
-/*  SafeAdd: adds a value and test whether overflow/underflow occured. */
+/*  SafeAdd: adds a value and test whether overflow/underflow occurred. */
 /***********************************************************************/
 template <class TYPE>
 TYPE TYPVAL<TYPE>::SafeAdd(TYPE n1, TYPE n2)
@@ -1000,12 +1053,12 @@ TYPE TYPVAL<TYPE>::SafeAdd(TYPE n1, TYPE n2)
   if ((n2 > 0) && (n < n1)) {
     // Overflow
     strcpy(g->Message, MSG(FIX_OVFLW_ADD));
-    longjmp(g->jumper[g->jump_level], 138);
-  } else if ((n2 < 0) && (n > n1)) {
+		throw 138;
+	} else if ((n2 < 0) && (n > n1)) {
     // Underflow
     strcpy(g->Message, MSG(FIX_UNFLW_ADD));
-    longjmp(g->jumper[g->jump_level], 138);
-  } // endif's n2
+		throw 138;
+	} // endif's n2
 
   return n;
   } // end of SafeAdd
@@ -1017,7 +1070,7 @@ inline double TYPVAL<double>::SafeAdd(double n1, double n2)
   } // end of SafeAdd
 
 /***********************************************************************/
-/*  SafeMult: multiply values and test whether overflow occured.       */
+/*  SafeMult: multiply values and test whether overflow occurred.       */
 /***********************************************************************/
 template <class TYPE>
 TYPE TYPVAL<TYPE>::SafeMult(TYPE n1, TYPE n2)
@@ -1028,12 +1081,12 @@ TYPE TYPVAL<TYPE>::SafeMult(TYPE n1, TYPE n2)
   if (n > MinMaxVal(true)) {
     // Overflow
     strcpy(g->Message, MSG(FIX_OVFLW_TIMES));
-    longjmp(g->jumper[g->jump_level], 138);
-  } else if (n < MinMaxVal(false)) {
+		throw 138;
+	} else if (n < MinMaxVal(false)) {
     // Underflow
     strcpy(g->Message, MSG(FIX_UNFLW_TIMES));
-    longjmp(g->jumper[g->jump_level], 138);
-  } // endif's n2
+		throw 138;
+	} // endif's n2
 
   return (TYPE)n;
   } // end of SafeMult
@@ -1151,7 +1204,7 @@ bool TYPVAL<TYPE>::Compall(PGLOBAL g, PVAL *vp, int np, OPVAL op)
 /*  This function assumes that the format matches the value type.      */
 /***********************************************************************/
 template <class TYPE>
-bool TYPVAL<TYPE>::FormatValue(PVAL vp, char *fmt)
+bool TYPVAL<TYPE>::FormatValue(PVAL vp, PCSZ fmt)
   {
   char *buf = (char*)vp->GetTo_Val();        // Should be big enough
   int   n = sprintf(buf, fmt, Tval);
@@ -1173,37 +1226,6 @@ bool TYPVAL<TYPE>::SetConstFormat(PGLOBAL g, FORMAT& fmt)
   return false;
   } // end of SetConstFormat
 
-/***********************************************************************/
-/*  Make file output of a typed object.                                */
-/***********************************************************************/
-template <class TYPE>
-void TYPVAL<TYPE>::Print(PGLOBAL g, FILE *f, uint n)
-  {
-  char m[64], buf[12];
-
-  memset(m, ' ', n);                             /* Make margin string */
-  m[n] = '\0';
-
-  if (Null)
-    fprintf(f, "%s<null>\n", m);
-  else
-    fprintf(f, strcat(strcat(strcpy(buf, "%s"), Fmt), "\n"), m, Tval);
-
-  } /* end of Print */
-
-/***********************************************************************/
-/*  Make string output of a int object.                                */
-/***********************************************************************/
-template <class TYPE>
-void TYPVAL<TYPE>::Print(PGLOBAL g, char *ps, uint z)
-  {
-  if (Null)
-    strcpy(ps, "<null>");
-  else
-    sprintf(ps, Fmt, Tval);
-
-  } /* end of Print */
-
 /* -------------------------- Class STRING --------------------------- */
 
 /***********************************************************************/
@@ -1329,7 +1351,7 @@ bool TYPVAL<PSZ>::SetValue_pval(PVAL valp, bool chktype)
 
     char buf[64];
 
-    if (!(Null = valp->IsNull() && Nullable))
+    if (!(Null = (valp->IsNull() && Nullable)))
       strncpy(Strp, valp->GetCharString(buf), Len);
     else
       Reset();
@@ -1342,22 +1364,25 @@ bool TYPVAL<PSZ>::SetValue_pval(PVAL valp, bool chktype)
 /***********************************************************************/
 /*  STRING SetValue: fill string with chars extracted from a line.     */
 /***********************************************************************/
-bool TYPVAL<PSZ>::SetValue_char(char *p, int n)
+bool TYPVAL<PSZ>::SetValue_char(const char *cp, int n)
   {
-  bool rc;
+  bool rc = false;
 
-  if (p && n > 0) {
-    rc = n > Len;
+  if (!cp || n == 0) {
+		Reset();
+		Null = (cp) ? false : Nullable;
+	} else if (cp != Strp) {
+		const char *p = cp + n - 1;
+
+		for (p; p >= cp; p--, n--)
+			if (*p && *p != ' ')
+				break;
+
+		rc = n > Len;
 
     if ((n = MY_MIN(n, Len))) {
-    	strncpy(Strp, p, n);
-
-//    for (p = Strp + n - 1; p >= Strp && (*p == ' ' || *p == '\0'); p--) ;
-      for (p = Strp + n - 1; p >= Strp; p--)
-        if (*p && *p != ' ')
-          break;
-
-      *(++p) = '\0';
+    	strncpy(Strp, cp, n);
+	    Strp[n] = '\0';
 
       if (trace > 1)
         htrc(" Setting string to: '%s'\n", Strp);
@@ -1366,10 +1391,6 @@ bool TYPVAL<PSZ>::SetValue_char(char *p, int n)
       Reset();
 
     Null = false;
-  } else {
-    rc = false;
-    Reset();
-    Null = Nullable;
   } // endif p
 
   return rc;
@@ -1378,14 +1399,14 @@ bool TYPVAL<PSZ>::SetValue_char(char *p, int n)
 /***********************************************************************/
 /*  STRING SetValue: fill string with another string.                  */
 /***********************************************************************/
-void TYPVAL<PSZ>::SetValue_psz(PSZ s)
+void TYPVAL<PSZ>::SetValue_psz(PCSZ s)
   {
-  if (s) {
-    strncpy(Strp, s, Len);
+  if (!s) {
+		Reset();
+		Null = Nullable;
+	} else if (s != Strp) {
+		strncpy(Strp, s, Len);
     Null = false;
-  } else {
-    Reset();
-    Null = Nullable;
   } // endif s
 
   } // end of SetValue_psz
@@ -1414,8 +1435,8 @@ void TYPVAL<PSZ>::SetValue(int n)
 
   if (k > Len) {
     sprintf(g->Message, MSG(VALSTR_TOO_LONG), buf, Len);
-    longjmp(g->jumper[g->jump_level], 138);
-  } else
+		throw 138;
+	} else
     SetValue_psz(buf);
 
   Null = false;
@@ -1432,7 +1453,7 @@ void TYPVAL<PSZ>::SetValue(uint n)
 
   if (k > Len) {
     sprintf(g->Message, MSG(VALSTR_TOO_LONG), buf, Len);
-    longjmp(g->jumper[g->jump_level], 138);
+    throw 138;
   } else
     SetValue_psz(buf);
 
@@ -1468,8 +1489,8 @@ void TYPVAL<PSZ>::SetValue(longlong n)
 
   if (k > Len) {
     sprintf(g->Message, MSG(VALSTR_TOO_LONG), buf, Len);
-    longjmp(g->jumper[g->jump_level], 138);
-  } else
+		throw 138;
+	} else
     SetValue_psz(buf);
 
   Null = false;
@@ -1486,7 +1507,7 @@ void TYPVAL<PSZ>::SetValue(ulonglong n)
 
   if (k > Len) {
     sprintf(g->Message, MSG(VALSTR_TOO_LONG), buf, Len);
-    longjmp(g->jumper[g->jump_level], 138);
+    throw 138;
   } else
     SetValue_psz(buf);
 
@@ -1511,8 +1532,8 @@ void TYPVAL<PSZ>::SetValue(double f)
 
   if (k > Len) {
     sprintf(g->Message, MSG(VALSTR_TOO_LONG), buf, Len);
-    longjmp(g->jumper[g->jump_level], 138);
-  } else
+		throw 138;
+	} else
     SetValue_psz(buf);
 
   Null = false;
@@ -1541,7 +1562,7 @@ void TYPVAL<PSZ>::SetValue(uchar c)
 /***********************************************************************/
 void TYPVAL<PSZ>::SetBinValue(void *p)
   {
-  SetValue_char((char *)p, Len);
+  SetValue_char((const char *)p, Len);
   } // end of SetBinValue
 
 /***********************************************************************/
@@ -1636,32 +1657,36 @@ bool TYPVAL<PSZ>::Compute(PGLOBAL g, PVAL *vp, int np, OPVAL op)
   int   i;
 
   for (i = 0; i < np; i++)
-    p[i] = vp[i]->GetCharString(val[i]);
+    p[i] = vp[i]->IsNull() ? NULL : vp[i]->GetCharString(val[i]);
 
-  switch (op) {
-    case OP_CNC:
-      assert(np == 1 || np == 2);
+	if (p[i]) {
+		switch (op) {
+			case OP_CNC:
+				assert(np == 1 || np == 2);
 
-      if (np == 2)
-        strncpy(Strp, p[0], Len);
+				if (np == 2)
+					SetValue_psz(p[0]);
 
-      if ((i = Len - (signed)strlen(Strp)) > 0)
-        strncat(Strp, p[np - 1], i);
+				if ((i = Len - (signed)strlen(Strp)) > 0)
+					strncat(Strp, p[np - 1], i);
 
-      break;
-    case OP_MIN:
-      assert(np == 2);
-      strcpy(Strp, (strcmp(p[0], p[1]) < 0) ? p[0] : p[1]);
-      break;
-    case OP_MAX:
-      assert(np == 2);
-      strcpy(Strp, (strcmp(p[0], p[1]) > 0) ? p[0] : p[1]);
-      break;
-    default:
-//    sprintf(g->Message, MSG(BAD_EXP_OPER), op);
-      strcpy(g->Message, "Function not supported");
-      return true;
-    } // endswitch op
+				break;
+			case OP_MIN:
+				assert(np == 2);
+				SetValue_psz((strcmp(p[0], p[1]) < 0) ? p[0] : p[1]);
+				break;
+			case OP_MAX:
+				assert(np == 2);
+				SetValue_psz((strcmp(p[0], p[1]) > 0) ? p[0] : p[1]);
+				break;
+			default:
+				//    sprintf(g->Message, MSG(BAD_EXP_OPER), op);
+				strcpy(g->Message, "Function not supported");
+				return true;
+		} // endswitch op
+
+		Null = false;
+	} // endif p[i]
 
   return false;
   } // end of Compute
@@ -1671,7 +1696,7 @@ bool TYPVAL<PSZ>::Compute(PGLOBAL g, PVAL *vp, int np, OPVAL op)
 /*  constructed from its own value formated using the fmt format.      */
 /*  This function assumes that the format matches the value type.      */
 /***********************************************************************/
-bool TYPVAL<PSZ>::FormatValue(PVAL vp, char *fmt)
+bool TYPVAL<PSZ>::FormatValue(PVAL vp, PCSZ fmt)
   {
   char *buf = (char*)vp->GetTo_Val();        // Should be big enough
   int   n = sprintf(buf, fmt, Strp);
@@ -1690,6 +1715,18 @@ bool TYPVAL<PSZ>::SetConstFormat(PGLOBAL, FORMAT& fmt)
   return false;
   } // end of SetConstFormat
 
+/***********************************************************************/
+/*  Make string output of an object value.                             */
+/***********************************************************************/
+void TYPVAL<PSZ>::Prints(PGLOBAL g, char *ps, uint z)
+{
+	if (Null)
+		strncpy(ps, "null", z);
+	else
+		strcat(strncat(strncpy(ps, "\"", z), Strp, z-2), "\"");
+
+} // end of Prints
+
 /* -------------------------- Class DECIMAL -------------------------- */
 
 /***********************************************************************/
@@ -1779,102 +1816,6 @@ bool DECVAL::GetBinValue(void *buf, int buflen, bool go)
   return false;
   } // end of GetBinValue
 
-#if 0
-/***********************************************************************/
-/*  DECIMAL SetValue: copy the value of another Value object.          */
-/***********************************************************************/
-bool DECVAL::SetValue_pval(PVAL valp, bool chktype)
-  {
-  if (chktype && (valp->GetType() != Type || valp->GetSize() > Len))
-    return true;
-
-  char buf[64];
-
-  if (!(Null = valp->IsNull() && Nullable))
-    strncpy(Strp, valp->GetCharString(buf), Len);
-  else
-    Reset();
-
-  return false;
-  } // end of SetValue_pval
-
-/***********************************************************************/
-/*  DECIMAL SetValue: fill string with chars extracted from a line.    */
-/***********************************************************************/
-bool DECVAL::SetValue_char(char *p, int n)
-  {
-  bool rc;
-
-  if (p && n > 0) {
-    rc = n > Len;
-
-    if ((n = MY_MIN(n, Len))) {
-    	strncpy(Strp, p, n);
-
-//    for (p = Strp + n - 1; p >= Strp && (*p == ' ' || *p == '\0'); p--) ;
-      for (p = Strp + n - 1; p >= Strp; p--)
-        if (*p && *p != ' ')
-          break;
-
-      *(++p) = '\0';
-
-      if (trace > 1)
-        htrc(" Setting string to: '%s'\n", Strp);
-
-    } else
-      Reset();
-
-    Null = false;
-  } else {
-    rc = false;
-    Reset();
-    Null = Nullable;
-  } // endif p
-
-  return rc;
-  } // end of SetValue_char
-
-/***********************************************************************/
-/*  DECIMAL SetValue: fill string with another string.                 */
-/***********************************************************************/
-void DECVAL::SetValue_psz(PSZ s)
-  {
-  if (s) {
-    strncpy(Strp, s, Len);
-    Null = false;
-  } else {
-    Reset();
-    Null = Nullable;
-  } // endif s
-
-  } // end of SetValue_psz
-
-/***********************************************************************/
-/*  DECIMAL SetValue: fill string with a string extracted from a block.*/
-/***********************************************************************/
-void DECVAL::SetValue_pvblk(PVBLK blk, int n)
-  {
-  // STRBLK's can return a NULL pointer
-  SetValue_psz(blk->GetCharValue(n));
-  } // end of SetValue_pvblk
-
-/***********************************************************************/
-/*  DECIMAL SetBinValue: fill string with chars extracted from a line. */
-/***********************************************************************/
-void DECVAL::SetBinValue(void *p)
-  {
-  SetValue_char((char *)p, Len);
-  } // end of SetBinValue
-
-/***********************************************************************/
-/*  DECIMAL GetCharString: get string representation of a char value.  */
-/***********************************************************************/
-char *DECVAL::GetCharString(char *p)
-  {
-  return Strp;
-  } // end of GetCharString
-#endif // 0
-
 /***********************************************************************/
 /*  DECIMAL compare value with another Value.                          */
 /***********************************************************************/
@@ -1909,32 +1850,6 @@ int DECVAL::CompareValue(PVAL vp)
   return (f > n) ? 1 : (f < n) ? (-1) : 0;
   } // end of CompareValue
 
-#if 0
-/***********************************************************************/
-/*  FormatValue: This function set vp (a STRING value) to the string   */
-/*  constructed from its own value formated using the fmt format.      */
-/*  This function assumes that the format matches the value type.      */
-/***********************************************************************/
-bool DECVAL::FormatValue(PVAL vp, char *fmt)
-  {
-  char *buf = (char*)vp->GetTo_Val();        // Should be big enough
-  int   n = sprintf(buf, fmt, Strp);
-
-  return (n > vp->GetValLen());
-  } // end of FormatValue
-
-/***********************************************************************/
-/*  DECIMAL SetFormat function (used to set SELECT output format).     */
-/***********************************************************************/
-bool DECVAL::SetConstFormat(PGLOBAL g, FORMAT& fmt)
-  {
-  fmt.Type[0] = 'C';
-  fmt.Length = Len;
-  fmt.Prec = 0;
-  return false;
-  } // end of SetConstFormat
-#endif // 0
-
 /* -------------------------- Class BINVAL --------------------------- */
 
 /***********************************************************************/
@@ -1943,8 +1858,9 @@ bool DECVAL::SetConstFormat(PGLOBAL g, FORMAT& fmt)
 BINVAL::BINVAL(PGLOBAL g, void *p, int cl, int n) : VALUE(TYPE_BIN)
   {
   assert(g);
-  Len = n;
-  Clen = cl;
+//Len = n;
+	Len = (g) ? n : (p) ? strlen((char*)p) : 0;
+	Clen = cl;
 	Binp = PlugSubAlloc(g, NULL, Clen + 1);
   memset(Binp, 0, Clen + 1);
 
@@ -2077,10 +1993,15 @@ bool BINVAL::SetValue_pval(PVAL valp, bool chktype)
       return true;
 
     if (!(Null = valp->IsNull() && Nullable)) {
-      if ((rc = (Len = valp->GetSize()) > Clen))
+			int len = Len;
+
+			if ((rc = (Len = valp->GetSize()) > Clen))
         Len = Clen;
+			else if (len > Len)
+				memset(Binp, 0, len);
 
       memcpy(Binp, valp->GetTo_Val(), Len);
+			((char*)Binp)[Len] = 0;
     } else
       Reset();
 
@@ -2092,15 +2013,20 @@ bool BINVAL::SetValue_pval(PVAL valp, bool chktype)
 /***********************************************************************/
 /*  BINVAL SetValue: fill value with chars extracted from a line.      */
 /***********************************************************************/
-bool BINVAL::SetValue_char(char *p, int n)
+bool BINVAL::SetValue_char(const char *p, int n)
   {
   bool rc;
 
   if (p && n > 0) {
-    rc = n > Clen;
-    Len = MY_MIN(n, Clen);
-    memcpy(Binp, p, Len);
-    Null = false;
+		int len = Len;
+
+    if (len > (Len = MY_MIN(n, Clen)))
+			memset(Binp, 0, len);
+
+		memcpy(Binp, p, Len);
+		((char*)Binp)[Len] = 0;
+		rc = n > Clen;
+		Null = false;
   } else {
     rc = false;
     Reset();
@@ -2113,12 +2039,17 @@ bool BINVAL::SetValue_char(char *p, int n)
 /***********************************************************************/
 /*  BINVAL SetValue: fill value with another string.                   */
 /***********************************************************************/
-void BINVAL::SetValue_psz(PSZ s)
+void BINVAL::SetValue_psz(PCSZ s)
   {
   if (s) {
-    Len = MY_MIN(Clen, (signed)strlen(s));
-    memcpy(Binp, s, Len);
-    Null = false;
+		int len = Len;
+
+		if (len > (Len = MY_MIN(Clen, (signed)strlen(s))))
+			memset(Binp, 0, len);
+
+		memcpy(Binp, s, Len);
+		((char*)Binp)[Len] = 0;
+		Null = false;
   } else {
     Reset();
     Null = Nullable;
@@ -2138,14 +2069,19 @@ void BINVAL::SetValue_pvblk(PVBLK blk, int n)
     Reset();
     Null = Nullable;
   } else if (vp != Binp) {
+		int len = Len;
+
     if (blk->GetType() == TYPE_STRING)
       Len = strlen((char*)vp);
     else
       Len = blk->GetVlen();
 
-    Len = MY_MIN(Clen, Len);
+    if (len > (Len = MY_MIN(Clen, Len)))
+			memset(Binp, 0, len);
+
     memcpy(Binp, vp, Len);
-    Null = false;
+		((char*)Binp)[Len] = 0;
+		Null = false;
   } // endif vp
 
   } // end of SetValue_pvblk
@@ -2156,7 +2092,10 @@ void BINVAL::SetValue_pvblk(PVBLK blk, int n)
 void BINVAL::SetValue(int n)
   {
   if (Clen >= 4) {
-    *((int*)Binp) = n;
+		if (Len > 4)
+			memset(Binp, 0, Len);
+
+		*((int*)Binp) = n;
     Len = 4;
   } else
     SetValue((short)n);
@@ -2169,7 +2108,10 @@ void BINVAL::SetValue(int n)
 void BINVAL::SetValue(uint n)
   {
   if (Clen >= 4) {
-    *((uint*)Binp) = n;
+		if (Len > 4)
+			memset(Binp, 0, Len);
+
+		*((uint*)Binp) = n;
     Len = 4;
   } else
     SetValue((ushort)n);
@@ -2182,7 +2124,10 @@ void BINVAL::SetValue(uint n)
 void BINVAL::SetValue(short i)
   {
   if (Clen >= 2) {
-    *((int*)Binp) = i;
+		if (Len > 2)
+			memset(Binp, 0, Len);
+
+		*((int*)Binp) = i;
     Len = 2;
   } else
     SetValue((char)i);
@@ -2195,7 +2140,10 @@ void BINVAL::SetValue(short i)
 void BINVAL::SetValue(ushort i)
   {
   if (Clen >= 2) {
-    *((uint*)Binp) = i;
+		if (Len > 2)
+			memset(Binp, 0, Len);
+
+		*((uint*)Binp) = i;
     Len = 2;
   } else
     SetValue((uchar)i);
@@ -2208,7 +2156,10 @@ void BINVAL::SetValue(ushort i)
 void BINVAL::SetValue(longlong n)
   {
   if (Clen >= 8) {
-    *((longlong*)Binp) = n;
+		if (Len > 8)
+			memset(Binp, 0, Len);
+
+		*((longlong*)Binp) = n;
     Len = 8;
   } else
     SetValue((int)n);
@@ -2221,7 +2172,10 @@ void BINVAL::SetValue(longlong n)
 void BINVAL::SetValue(ulonglong n)
   {
   if (Clen >= 8) {
-    *((ulonglong*)Binp) = n;
+		if (Len > 8)
+			memset(Binp, 0, Len);
+
+		*((ulonglong*)Binp) = n;
     Len = 8;
   } else
     SetValue((uint)n);
@@ -2232,6 +2186,9 @@ void BINVAL::SetValue(ulonglong n)
 /***********************************************************************/
 void BINVAL::SetValue(double n)
   {
+	if (Len > 8)
+	  memset(Binp, 0, Len);
+
   if (Clen >= 8) {
     *((double*)Binp) = n;
     Len = 8;
@@ -2248,7 +2205,10 @@ void BINVAL::SetValue(double n)
 /***********************************************************************/
 void BINVAL::SetValue(char c)
   {
-  *((char*)Binp) = c;
+	if (Len > 1)
+		memset(Binp, 0, Len);
+
+	*((char*)Binp) = c;
   Len = 1;
   } // end of SetValue
 
@@ -2257,7 +2217,10 @@ void BINVAL::SetValue(char c)
 /***********************************************************************/
 void BINVAL::SetValue(uchar c)
   {
-  *((uchar*)Binp) = c;
+	if (Len > 1)
+		memset(Binp, 0, Len);
+
+	*((uchar*)Binp) = c;
   Len = 1;
   } // end of SetValue
 
@@ -2267,6 +2230,7 @@ void BINVAL::SetValue(uchar c)
 void BINVAL::SetBinValue(void *p)
   {
   memcpy(Binp, p, Clen);
+	Len = Clen;
   } // end of SetBinValue
 
 /***********************************************************************/
@@ -2292,10 +2256,11 @@ bool BINVAL::GetBinValue(void *buf, int buflen, bool go)
 /***********************************************************************/
 char *BINVAL::ShowValue(char *buf, int len)
   {
-  int n = MY_MIN(Len, len / 2);
+  //int n = MY_MIN(Len, len / 2);
 
-  sprintf(buf, GetXfmt(), n, Binp);
-  return buf;
+  //sprintf(buf, GetXfmt(), n, Binp);
+  //return buf;
+	return (char*)Binp;
   } // end of ShowValue
 
 /***********************************************************************/
@@ -2339,7 +2304,7 @@ bool BINVAL::IsEqual(PVAL vp, bool chktype)
 /*  constructed from its own value formated using the fmt format.      */
 /*  This function assumes that the format matches the value type.      */
 /***********************************************************************/
-bool BINVAL::FormatValue(PVAL vp, char *fmt)
+bool BINVAL::FormatValue(PVAL vp, PCSZ fmt)
   {
   char *buf = (char*)vp->GetTo_Val();        // Should be big enough
   int   n = sprintf(buf, fmt, Len, Binp);
@@ -2363,7 +2328,7 @@ bool BINVAL::SetConstFormat(PGLOBAL, FORMAT& fmt)
 /***********************************************************************/
 /*  DTVAL  public constructor for new void values.                     */
 /***********************************************************************/
-DTVAL::DTVAL(PGLOBAL g, int n, int prec, PSZ fmt)
+DTVAL::DTVAL(PGLOBAL g, int n, int prec, PCSZ fmt)
      : TYPVAL<int>((int)0, TYPE_DATE)
   {
   if (!fmt) {
@@ -2392,7 +2357,7 @@ DTVAL::DTVAL(int n) : TYPVAL<int>(n, TYPE_DATE)
 /***********************************************************************/
 /*  Set format so formatted dates can be converted on input/output.    */
 /***********************************************************************/
-bool DTVAL::SetFormat(PGLOBAL g, PSZ fmt, int len, int year)
+bool DTVAL::SetFormat(PGLOBAL g, PCSZ fmt, int len, int year)
   {
   Pdtp = MakeDateFormat(g, fmt, true, true, (year > 9999) ? 1 : 0);
   Sdate = (char*)PlugSubAlloc(g, NULL, len + 1);
@@ -2440,7 +2405,6 @@ void DTVAL::SetTimeShift(void)
 
   } // end of SetTimeShift
 
-#if defined(connect_EXPORTS)
 // Added by Alexander Barkov
 static void TIME_to_localtime(struct tm *tm, const MYSQL_TIME *ltime)
 {
@@ -2462,9 +2426,6 @@ static struct tm *gmtime_mysql(const time_t *timep, struct tm *tm)
   TIME_to_localtime(tm, &ltime);
   return tm;
 } // end of gmtime_mysql
-#else
-#define gmtime_mysql(T,B)			gmtime((const time_t *)T)
-#endif
 
 /***********************************************************************/
 /*  GetGmTime: returns a pointer to a static tm structure obtained     */
@@ -2493,7 +2454,6 @@ struct tm *DTVAL::GetGmTime(struct tm *tm_buffer)
   return datm;
   } // end of GetGmTime
 
-#if defined(connect_EXPORTS)
 // Added by Alexander Barkov
 static time_t mktime_mysql(struct tm *ptm)
 {
@@ -2504,9 +2464,6 @@ static time_t mktime_mysql(struct tm *ptm)
   time_t t= TIME_to_timestamp(current_thd, &ltime, &error_code);
   return error_code ? (time_t) -1 : t;
 }
-#else
-#define mktime_mysql   mktime
-#endif
 
 /***********************************************************************/
 /*  MakeTime: calculates a date value from a tm structures using the   */
@@ -2658,7 +2615,11 @@ bool DTVAL::SetValue_pval(PVAL valp, bool chktype)
 
         ndv = ExtractDate(valp->GetCharValue(), Pdtp, DefYear, dval);
         MakeDate(NULL, dval, ndv);
-      } else
+			} else if (valp->GetType() == TYPE_BIGINT &&
+				       !(valp->GetBigintValue() % 1000)) {
+				// Assuming that this timestamp is in milliseconds
+				Tval = (int)(valp->GetBigintValue() / 1000);
+			}	else
         Tval = valp->GetIntValue();
 
     } else
@@ -2672,14 +2633,14 @@ bool DTVAL::SetValue_pval(PVAL valp, bool chktype)
 /***********************************************************************/
 /*  SetValue: convert chars extracted from a line to date value.       */
 /***********************************************************************/
-bool DTVAL::SetValue_char(char *p, int n)
+bool DTVAL::SetValue_char(const char *p, int n)
   {
-  bool rc;
+  bool rc= 0;
 
   if (Pdtp) {
-    char *p2;
-    int   ndv;
-    int  dval[6];
+    const char *p2;
+    int ndv;
+    int dval[6];
 
 		if (n > 0) {
 			// Trim trailing blanks
@@ -2711,11 +2672,11 @@ bool DTVAL::SetValue_char(char *p, int n)
 /***********************************************************************/
 /*  SetValue: convert a char string to date value.                     */
 /***********************************************************************/
-void DTVAL::SetValue_psz(PSZ p)
+void DTVAL::SetValue_psz(PCSZ p)
   {
   if (Pdtp) {
-    int   ndv;
-    int  dval[6];
+    int ndv;
+    int dval[6];
 
     strncpy(Sdate, p, Len);
     Sdate[Len] = '\0';
@@ -2805,8 +2766,10 @@ char *DTVAL::ShowValue(char *buf, int len)
         strncat(p, "Error", m);
         } // endif n
 
-    } else
-      p = "";                 // DEFAULT VALUE ???
+		} else {
+			p = buf;
+			*p = '\0';               // DEFAULT VALUE ???
+		} // endif Null
 
     return p;
   } else
@@ -2871,7 +2834,7 @@ bool DTVAL::WeekNum(PGLOBAL g, int& nval)
 /*  constructed from its own value formated using the fmt format.      */
 /*  This function assumes that the format matches the value type.      */
 /***********************************************************************/
-bool DTVAL::FormatValue(PVAL vp, char *fmt)
+bool DTVAL::FormatValue(PVAL vp, PCSZ fmt)
   {
   char     *buf = (char*)vp->GetTo_Val();       // Should be big enough
   struct tm tm, *ptm = GetGmTime(&tm);
