@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (C) 2013, 2015, Google Inc. All Rights Reserved.
-Copyright (C) 2014, 2015, MariaDB Corporation. All Rights Reserved.
+Copyright (C) 2014, 2017, MariaDB Corporation. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -25,8 +25,7 @@ Modified           Jan Lindström jan.lindstrom@mariadb.com
 *******************************************************/
 #include "m_string.h"
 #include "log0crypt.h"
-#include <my_crypt.h>
-#include <my_crypt.h>
+#include <mysql/service_my_crypt.h>
 
 #include "log0log.h"
 #include "srv0start.h" // for srv_start_lsn
@@ -34,8 +33,9 @@ Modified           Jan Lindström jan.lindstrom@mariadb.com
 
 #include "ha_prototypes.h" // IB_LOG_
 
-#include "my_crypt.h"
 
+/* Used for debugging */
+// #define DEBUG_CRYPT 1
 #define UNENCRYPTED_KEY_VER	0
 
 /* If true, enable redo log encryption. */
@@ -97,16 +97,24 @@ get_crypt_info(
 {
 	/* so that no one is modifying array while we search */
 	ut_ad(mutex_own(&(log_sys->mutex)));
+	size_t items = crypt_info.size();
 
 	/* a log block only stores 4-bytes of checkpoint no */
 	checkpoint_no &= 0xFFFFFFFF;
-	for (size_t i = 0; i < crypt_info.size(); i++) {
+	for (size_t i = 0; i < items; i++) {
 		struct crypt_info_t* it = &crypt_info[i];
 
 		if (it->checkpoint_no == checkpoint_no) {
 			return it;
 		}
 	}
+
+	/* If checkpoint contains more than one key and we did not
+	find the correct one use the first one. */
+	if (items) {
+		return (&crypt_info[0]);
+	}
+
 	return NULL;
 }
 
@@ -117,11 +125,36 @@ static
 const crypt_info_t*
 get_crypt_info(
 /*===========*/
-	const byte* log_block) {
+	const byte* log_block)
+{
 	ib_uint64_t checkpoint_no = log_block_get_checkpoint_no(log_block);
 	return get_crypt_info(checkpoint_no);
 }
 
+/*********************************************************************//**
+Print checkpoint no from log block and all encryption keys from
+checkpoints if they are present. Used for problem analysis. */
+void
+log_crypt_print_checkpoint_keys(
+/*============================*/
+	const byte* log_block)
+{
+	ib_uint64_t checkpoint_no = log_block_get_checkpoint_no(log_block);
+
+	if (crypt_info.size()) {
+		fprintf(stderr,
+			"InnoDB: redo log checkpoint: " UINT64PF " [ chk key ]: ",
+			checkpoint_no);
+		for (size_t i = 0; i < crypt_info.size(); i++) {
+			struct crypt_info_t* it = &crypt_info[i];
+			fprintf(stderr, "[ " UINT64PF " %u ] ",
+				it->checkpoint_no,
+				it->key_version);
+		}
+		fprintf(stderr, "\n");
+	}
+}
+
 /*********************************************************************//**
 Call AES CTR to encrypt/decrypt log blocks. */
 static
@@ -131,7 +164,8 @@ log_blocks_crypt(
 	const byte* block,	/*!< in: blocks before encrypt/decrypt*/
 	ulint size,		/*!< in: size of block */
 	byte* dst_block,	/*!< out: blocks after encrypt/decrypt */
-	int what)		/*!< in: encrypt or decrypt*/
+	int what,		/*!< in: encrypt or decrypt*/
+	const crypt_info_t* crypt_info) /*!< in: crypt info or NULL */
 {
 	byte *log_block = (byte*)block;
 	Crypt_result rc = MY_AES_OK;
@@ -146,7 +180,8 @@ log_blocks_crypt(
 		lsn_t log_block_start_lsn = log_block_get_start_lsn(
 			lsn, log_block_no);
 
-		const crypt_info_t* info = get_crypt_info(log_block);
+		const crypt_info_t* info = crypt_info == NULL ? get_crypt_info(log_block) :
+			crypt_info;
 #ifdef DEBUG_CRYPT
 		fprintf(stderr,
 			"%s %lu chkpt: %lu key: %u lsn: %lu\n",
@@ -156,12 +191,21 @@ log_blocks_crypt(
 			info ? info->key_version : 0,
 			log_block_start_lsn);
 #endif
+		/* If no key is found from checkpoint assume the log_block
+		to be unencrypted. If checkpoint contains the encryption key
+		compare log_block current checksum, if checksum matches,
+		block can't be encrypted. */
 		if (info == NULL ||
-		    info->key_version == UNENCRYPTED_KEY_VER) {
+		    info->key_version == UNENCRYPTED_KEY_VER ||
+			(log_block_checksum_is_ok_or_old_format(log_block, false) &&
+			 what == ENCRYPTION_FLAG_DECRYPT)) {
 			memcpy(dst_block, log_block, OS_FILE_LOG_BLOCK_SIZE);
 			goto next;
 		}
 
+		ut_ad(what == ENCRYPTION_FLAG_DECRYPT ? !log_block_checksum_is_ok_or_old_format(log_block, false) :
+			log_block_checksum_is_ok_or_old_format(log_block, false));
+
 		// Assume log block header is not encrypted
 		memcpy(dst_block, log_block, LOG_BLOCK_HDR_SIZE);
 
@@ -193,6 +237,129 @@ next:
 	return rc;
 }
 
+/** Encrypt/decrypt temporary log blocks.
+
+@param[in]	src_block	block to encrypt or decrypt
+@param[in]	size		size of the block
+@param[out]	dst_block	destination block
+@param[in]	what		ENCRYPTION_FLAG_ENCRYPT or
+				ENCRYPTION_FLAG_DECRYPT
+@param[in]	offs		offset to block
+@param[in]	space_id	tablespace id
+@return true if successfull, false in case of failure
+*/
+static
+bool
+log_tmp_blocks_crypt(
+	const byte*		src_block,
+	ulint			size,
+	byte*			dst_block,
+	int			what,
+	os_offset_t		offs,
+	ulint			space_id)
+{
+	Crypt_result rc = MY_AES_OK;
+	uint dst_len;
+	byte aes_ctr_counter[MY_AES_BLOCK_SIZE];
+	byte is_encrypt= what == ENCRYPTION_FLAG_ENCRYPT;
+	const crypt_info_t* info = static_cast<const crypt_info_t*>(&crypt_info[0]);
+
+	// AES_CTR_COUNTER = space_id + offs
+
+	bzero(aes_ctr_counter, MY_AES_BLOCK_SIZE);
+	mach_write_to_8(aes_ctr_counter, space_id);
+	mach_write_to_8(aes_ctr_counter + 8, offs);
+
+	rc = encryption_crypt(src_block, size,
+				dst_block, &dst_len,
+				(unsigned char*)(info->crypt_key), 16,
+				aes_ctr_counter, MY_AES_BLOCK_SIZE,
+				what | ENCRYPTION_FLAG_NOPAD,
+				LOG_DEFAULT_ENCRYPTION_KEY,
+				info->key_version);
+
+	if (rc != MY_AES_OK) {
+		ib_logf(IB_LOG_LEVEL_ERROR,
+			"%s failed for temporary log file with rc = %d",
+			is_encrypt ? "Encryption" : "Decryption",
+			rc);
+		return false;
+	}
+
+	return true;
+}
+
+/** Get crypt info
+@return pointer to log crypt info or NULL
+*/
+inline
+const crypt_info_t*
+get_crypt_info()
+{
+	mutex_enter(&log_sys->mutex);
+	const crypt_info_t* info = get_crypt_info(log_sys->next_checkpoint_no);
+	mutex_exit(&log_sys->mutex);
+
+	return info;
+}
+
+/** Find out is temporary log files encrypted.
+@return true if temporary log file should be encrypted, false if not */
+UNIV_INTERN
+bool
+log_tmp_is_encrypted()
+{
+	const crypt_info_t* info = get_crypt_info();
+
+	if (info == NULL || info->key_version == UNENCRYPTED_KEY_VER) {
+		return false;
+	}
+
+	return true;
+}
+
+/** Encrypt temporary log block.
+@param[in]	src_block	block to encrypt or decrypt
+@param[in]	size		size of the block
+@param[out]	dst_block	destination block
+@param[in]	offs		offset to block
+@param[in]	space_id	tablespace id
+@return true if successfull, false in case of failure
+*/
+UNIV_INTERN
+bool
+log_tmp_block_encrypt(
+	const byte*		src_block,
+	ulint			size,
+	byte*			dst_block,
+	os_offset_t		offs,
+	ulint			space_id)
+{
+	return (log_tmp_blocks_crypt(src_block, size, dst_block,
+				ENCRYPTION_FLAG_ENCRYPT, offs, space_id));
+}
+
+/** Decrypt temporary log block.
+@param[in]	src_block	block to encrypt or decrypt
+@param[in]	size		size of the block
+@param[out]	dst_block	destination block
+@param[in]	offs		offset to block
+@param[in]	space_id	tablespace id
+@return true if successfull, false in case of failure
+*/
+UNIV_INTERN
+bool
+log_tmp_block_decrypt(
+	const byte*		src_block,
+	ulint			size,
+	byte*			dst_block,
+	os_offset_t		offs,
+	ulint			space_id)
+{
+	return (log_tmp_blocks_crypt(src_block, size, dst_block,
+				     ENCRYPTION_FLAG_DECRYPT, offs, space_id));
+}
+
 /*********************************************************************//**
 Generate crypt key from crypt msg.
 @return true if successfull, false if not. */
@@ -209,30 +376,32 @@ init_crypt_key(
 		return true;
 	}
 
-	byte mysqld_key[MY_AES_BLOCK_SIZE] = {0};
+	byte mysqld_key[MY_AES_MAX_KEY_LENGTH] = {0};
 	uint keylen= sizeof(mysqld_key);
+	uint rc;
 
-	if (encryption_key_get(LOG_DEFAULT_ENCRYPTION_KEY, info->key_version, mysqld_key, &keylen))
-	{
+	rc = encryption_key_get(LOG_DEFAULT_ENCRYPTION_KEY, info->key_version, mysqld_key, &keylen);
+
+	if (rc) {
 		ib_logf(IB_LOG_LEVEL_ERROR,
 			"Redo log crypto: getting mysqld crypto key "
-			"from key version failed. Reason could be that requested"
+			"from key version failed err = %u. Reason could be that requested"
 			" key_version %u is not found or required encryption "
-			" key management is not found.", info->key_version);
+			" key management is not found.", rc, info->key_version);
 		return false;
 	}
 
 	uint dst_len;
-	int rc= my_aes_crypt(MY_AES_ECB, ENCRYPTION_FLAG_NOPAD|ENCRYPTION_FLAG_ENCRYPT,
+	int err= my_aes_crypt(MY_AES_ECB, ENCRYPTION_FLAG_NOPAD|ENCRYPTION_FLAG_ENCRYPT,
                              info->crypt_msg, sizeof(info->crypt_msg), //src, srclen
                              info->crypt_key, &dst_len, //dst, &dstlen
                              (unsigned char*)&mysqld_key, sizeof(mysqld_key),
                              NULL, 0);
 
-	if (rc != MY_AES_OK || dst_len != MY_AES_BLOCK_SIZE) {
+	if (err != MY_AES_OK || dst_len != MY_AES_BLOCK_SIZE) {
 		fprintf(stderr,
 			"\nInnodb redo log crypto: getting redo log crypto key "
-			"failed.\n");
+			"failed err = %d len = %u.\n", err, dst_len);
 		return false;
 	}
 
@@ -255,12 +424,22 @@ Add crypt info to set if it is not already present
 @return true if successfull, false if not- */
 static
 bool
-add_crypt_info(crypt_info_t* info)
+add_crypt_info(
+/*===========*/
+	crypt_info_t*	info,		/*!< in: crypt info */
+	bool		checkpoint_read)/*!< in: do we read checkpoint */
 {
+	const crypt_info_t* found=NULL;
 	/* so that no one is searching array while we modify it */
 	ut_ad(mutex_own(&(log_sys->mutex)));
 
-	if (get_crypt_info(info->checkpoint_no) != NULL) {
+	found = get_crypt_info(info->checkpoint_no);
+
+	/* If one crypt info is found then we add a new one only if we
+	are reading checkpoint from the log. New checkpoints will always
+	use the first created crypt info. */
+	if (found != NULL &&
+		( found->checkpoint_no == info->checkpoint_no || !checkpoint_read)) {
 		// already present...
 		return true;
 	}
@@ -290,7 +469,7 @@ log_blocks_encrypt(
 	const ulint size,		/*!< in: size of blocks, must be multiple of a log block */
 	byte* dst_block)		/*!< out: blocks after encryption */
 {
-	return log_blocks_crypt(block, size, dst_block, ENCRYPTION_FLAG_ENCRYPT);
+	return log_blocks_crypt(block, size, dst_block, ENCRYPTION_FLAG_ENCRYPT, NULL);
 }
 
 /*********************************************************************//**
@@ -333,7 +512,7 @@ log_crypt_set_ver_and_key(
 
 	}
 
-	add_crypt_info(&info);
+	add_crypt_info(&info, false);
 }
 
 /********************************************************
@@ -353,14 +532,16 @@ log_encrypt_before_write(
 		return;
 	}
 
-	if (info->key_version == UNENCRYPTED_KEY_VER) {
+	/* If the key is not encrypted or user has requested not to
+	encrypt, do not change log block. */
+	if (info->key_version == UNENCRYPTED_KEY_VER || !srv_encrypt_log) {
 		return;
 	}
 
 	byte* dst_frame = (byte*)malloc(size);
 
 	//encrypt log blocks content
-	Crypt_result result = log_blocks_crypt(block, size, dst_frame, ENCRYPTION_FLAG_ENCRYPT);
+	Crypt_result result = log_blocks_crypt(block, size, dst_frame, ENCRYPTION_FLAG_ENCRYPT, NULL);
 
 	if (result == MY_AES_OK) {
 		ut_ad(block[0] == dst_frame[0]);
@@ -386,7 +567,7 @@ log_decrypt_after_read(
 	byte* dst_frame = (byte*)malloc(size);
 
 	// decrypt log blocks content
-	Crypt_result result = log_blocks_crypt(frame, size, dst_frame, ENCRYPTION_FLAG_DECRYPT);
+	Crypt_result result = log_blocks_crypt(frame, size, dst_frame, ENCRYPTION_FLAG_DECRYPT, NULL);
 
 	if (result == MY_AES_OK) {
 		memcpy(frame, dst_frame, size);
@@ -489,7 +670,7 @@ log_crypt_read_checkpoint_buf(
 		memcpy(info.crypt_msg, buf + 8, MY_AES_BLOCK_SIZE);
 		memcpy(info.crypt_nonce, buf + 24, MY_AES_BLOCK_SIZE);
 
-		if (!add_crypt_info(&info)) {
+		if (!add_crypt_info(&info, true)) {
 			return false;
 		}
 		buf += LOG_CRYPT_ENTRY_SIZE;
